/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "quaqut.h"


int quaqut_cmdlineparam_process( int argc , char *argv[] )
{
  int result_code = quaqut_cmdlineparam_parse( argc , argv );
  
  if ( result_code < 0 ) /* an error occurred */
  {
    if ( result_code == QUAQUT_ERROR_DISPLAYVERSION )
    {
      quaqut_display_version();
    }
    else
    {
      quaqut_display_error( result_code , NULL , NULL );
      quaqut_display_instructions();
    }
    return 0;
  }
  else return 1; /* everything is ok */
}


int quaqut_cmdlineparam_parse( int argc , char *argv[] )
{
  /* First, we check if one or more -parameters have been passed in
     command line; syntax is simple: quaqut [-param [argument]] hostname */
  int i, cset = 0;
  
  for ( i = 1 ; i < argc ; i++ )
  {
    if ( argv[i][0] == '-' ) /* then this is a parameter */
    {
      switch ( argv[i][1] )
      {
        case 'a':
	case 's':
	case 'g':
	case 'y':
	  if ( !cset ) { quaqut.command = argv[i][1]; cset = 1; }
	  break;
	
	case 'r':
	  quaqut.options |= QUAQUT_OPTION_RAWINFO;
	  break;
	
	case 'v':
	  quaqut.options |= QUAQUT_OPTION_VERSION;
	  return QUAQUT_ERROR_DISPLAYVERSION;
	  break;
	
	case 't':
	{
	  int itimeout = 0;
	  if ( quaqut_cmdlineparam_checkparamarg( i , argc , argv ) )
	  {
	    itimeout = atoi( argv[i+1] );
	    /* Ensure that itimeout is a number in the valid interval */
	    if ( ( itimeout > 0 ) && ( itimeout <= QUAQUT_QUERY_MAX_TIMEOUT ) )
	    {
	      float ftimeout = (float)itimeout / 10;
	      quaqut.recv_timeout_sec = (int)ftimeout;
	      quaqut.recv_timeout_decsec = (ftimeout * 10) - ((int)ftimeout * 10);
	    }
	    else return QUAQUT_ERROR_INVALIDQUERYTIMEOUT; /* Wrong timeouts are not accepted */
	  }
	  else return QUAQUT_ERROR_MISSINGQUERYTIMEOUT;
	  i++;
	  break;
	}
	
	case 'd':
	{
	  int itimeout = 0;  
	  if ( quaqut_cmdlineparam_checkparamarg( i , argc , argv ) )
	  {
	    itimeout = atoi( argv[i+1] );
	    /* Ensure that itimeout is a number in the valid interval */
	    if ( ( itimeout >= QUAQUT_DAEMON_MIN_INTERVAL ) && ( itimeout <= QUAQUT_DAEMON_MAX_INTERVAL ) )
	    {
	      quaqut.daemon_interval_sec = itimeout;
	    }
	    else return QUAQUT_ERROR_INVALIDDAEMONINTERVAL; /* Wrong intervals are not accepted */	    
	    quaqut.options |= QUAQUT_OPTION_DAEMON;
	  }
	  else return QUAQUT_ERROR_MISSINGDAEMONINTERVAL;
	  i++;
	  break;
	}
	
	case 'p':
	case 'q':
	{
	  if ( quaqut_cmdlineparam_checkparamarg( i , argc , argv ) )
	  {
	    if ( argv[i][1] == 'p' ) quaqut.server_port = (unsigned short int)atoi( argv[i+1] ) + 1;
	    else quaqut.server_port = (unsigned short int)atoi( argv[i+1] );
	  }
	  else return QUAQUT_ERROR_MISSINGPORT;
	  i++;
	  break;
	}
	
	case 'f':
	{
	  if ( quaqut_cmdlineparam_checkparamarg( i , argc , argv ) )
	  {
	    quaqut.options |= QUAQUT_OPTION_LOGFILE;
	    quaqut.output_filename = argv[i+1];
	  }
	  else return QUAQUT_ERROR_MISSINGFILE;
	  i++;
	  break;
	}
      }
    }
    else /* it must be the hostname */
    {
        quaqut.server_host = argv[i];
	i = argc;  /* ignore any further data after hostname */
    }
  }
  
  if ( !cset ) { quaqut.command = QUAQUT_QUERY_STANDARD_COMMAND; }
  if ( quaqut.server_port == 0 ) quaqut.server_port = QUAQUT_QUERY_STANDARD_PORT;
  if ( quaqut.server_host == NULL ) return QUAQUT_ERROR_MISSINGHOSTNAME;
  else return 1;
}

int quaqut_cmdlineparam_checkparamarg( int i , int argc , char *argv[] )
{
  /* Ensure that the parameter argument has been specified after -param (example: -p 7777) */
  if ( i == (argc - 1) ) return 0;
  if ( argv[i+1][0] == '-' ) return 0;
  return 1;
}
