/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "quaqut.h"
#include "protocol.h"

int main(int argc, char *argv[])
{
  quaqut_utils_initvalues();

  /* Get information from cmdline parameters */
  if ( !quaqut_cmdlineparam_process( argc , argv ) ) return EXIT_FAILURE;
  
  /* Socket init */
  quaqut_network_initsocket();
  
  /* After socket init, register proper cleanup function */
  atexit( quaqut_network_shutsocket );
  signal( SIGTERM , quaqut_utils_cleanup );
  signal( SIGINT , quaqut_utils_cleanup );
  
  /* Hostname lookup */
  if ( !quaqut_network_lookup() ) return EXIT_FAILURE;
  
  if ( quaqut.options & QUAQUT_OPTION_DAEMON ) /* go with daemon mode */
  {
    #ifdef _WIN_32_
      quaqut.daemon_interval_sec = quaqut.daemon_interval_sec * 1000;
    #endif
    while ( 1 )
    {
      /* Ready; we can send our query and get our response, which
         depends on parameter specified by user in command line */
      quaqut_network_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command );
      /* Now we must translate buffer data in human readable form */
      quaqut_protocol_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command );
      /* Last, we have to display translated information */
      if ( !quaqut_display_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command ) ) return EXIT_FAILURE;
      quaqut_utils_cleanbuffers();
      sleep( quaqut.daemon_interval_sec );
    }
  }
  else /* just query once */
  {
    quaqut_network_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command );
    quaqut_protocol_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command );
    if ( !quaqut_display_process( QUAQUT_PROTOCOL_UT2004 , quaqut.command ) ) return EXIT_FAILURE;
  }
  
  return EXIT_SUCCESS;
}
