/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "quaqut.h"
#include "protocol.h"

void quaqut_protocol_process( int protocol , char command )
{
  switch ( command )
  {
    case 'a':
      quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
      quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEINFO ); 
      if ( quaqut.buffer_reply_size[QUAQUT_PLAYERINFO] > 0 )
        quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_PLAYERINFO );
      break;
    
    case 's':
      quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_SERVERINFO );
      break;
    
    case 'g':
      quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_GAMEINFO );
      break;
    
    case 'y':
      quaqut_protocol_translate( QUAQUT_PROTOCOL_UT2004 , QUAQUT_PLAYERINFO );
      break;
  }
}


void quaqut_protocol_translate( int protocol , int request )
{
  switch ( protocol )
  {
    case QUAQUT_PROTOCOL_UT2004:
      quaqut_protocol_ut2004( request );
  }
}


void quaqut_protocol_ut2004( int request )
{
  switch ( request )
  {
    case QUAQUT_SERVERINFO:
    {
      if ( quaqut.buffer_reply_size[request] == 0 )
      {
        /* Information has not been received correctly */
	quaqut.buffer_reply_entriesnum[request] = 0;
      }
      else
      {
        /* Server information from UT2004 (the most non-linear message) */
        unsigned char buffer[3072] = "";
        int index = 0;
      
        /* We can happily ignore bytes from 0 to 4 */
        index = 5;
      
        /* The next four bytes contain server ID, it seems this
           is always unused but we'll collect everything anyway */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );

        index += 4;
        /* Subsequent bytes should contain server IP, with length stored in the first byte;
           it seems this is never set and so the first byte has value zero; check it anyway */
        quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
      
        index += quaqut.buffer_reply[request][index] + 1;    
        /* Subsequent four bytes contain game port value */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;
        /* Subsequent four bytes should contain query port, but they are never filled in */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;    
        /* Subsequent bytes contain server name, with length stored in the first byte */
        quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
      
        index += quaqut.buffer_reply[request][index] + 1;
        /* Subsequent bytes contain game map name, with length stored in the first byte */
        quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
      
        index += quaqut.buffer_reply[request][index] + 1;
        /* Subsequent bytes contain game type name, with length stored in the first byte */
        quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
      
        index += quaqut.buffer_reply[request][index] + 1;
        /* Subsequent four bytes contain number of player currently in game */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;
        /* Subsequent four bytes contain max number of player allowed in game */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;
        /* Subsequent four bytes are for "ping" (always set to zero) */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;
        /* Subsequent four bytes are for server flags (always set to zero?) */
        quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
      
        index += 4;
        /* Subsequent bytes contain skill level, with length stored in the first byte */
        quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
      
        strncat( buffer , "\0" , 1 );     
      
        /* Ok, we did everything! Now, since we won't need raw UDP data anymore,
           we copy our translated output from temporary buffer to quaqut buffer */
        strncpy( quaqut.buffer_reply[request] , buffer , 2047 );
	quaqut.buffer_reply[request][2047] = '\0';
      
        /* We don't need to make a count of lines for serverinfo, they're always 12 */
        quaqut.buffer_reply_entriesnum[request] = 12;
      }
      
      break;
    }
    
    case QUAQUT_GAMEINFO:
    {
      if ( quaqut.buffer_reply_size[request] == 0 )
      {
        /* Information has not been received correctly */
	quaqut.buffer_reply_entriesnum[request] = 0;
      }
      else
      {
        /* Game information from UT2004 (the most linear message) */
        unsigned char buffer[3072] = "";
        int index = 0;
        int entrynum = 0;
        int max = quaqut.buffer_reply_size[request];
      
        /* We can happily ignore bytes from 0 to 4 */
        index = 5;
      
        /* This is easy, there are only strings */
        while ( index < max )
        {
          quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
	  index = index + quaqut.buffer_reply[request][index] + 1;
	  entrynum++;
        }
      
        strncat( buffer , "\0" , 1 ); 
      
        /* We copy our translated output from temporary buffer to quaqut buffer */
        strncpy( quaqut.buffer_reply[request] , buffer , 2047 );
	quaqut.buffer_reply[request][2047] = '\0';

        /* We need to make a count of lines for gameinfo, stored in entrynum */
        quaqut.buffer_reply_entriesnum[request] = entrynum;
      }
      
      break;
    }
    
    case QUAQUT_PLAYERINFO:
    {
      if ( quaqut.buffer_reply_size[request] == 0 )
      {
        /* Information has not been received correctly */
	quaqut.buffer_reply_entriesnum[request] = 0;
      }
      else
      {
        /* Player information from UT2004 (the quite-linear message) */
        unsigned char buffer[3072] = "";
        int index = 0;
        int entrynum = 0;
        int max = quaqut.buffer_reply_size[request];
      
        /* We already stripped the first five bytes when buffer was copied
           index = 5; */
      
        /* Even if there aren't only strings, there's a repeated pattern */
        while ( index < max )
        {
          /* Four bytes for player ID */
	  quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
	  index += 4;
	
	  /* String for player name */
	  quaqut_utils_bufferizestring( buffer , quaqut.buffer_reply[request] , index );
	  index += quaqut.buffer_reply[request][index] + 1;
	
	  /* Four bytes for player ping */
	  quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 0 );
	  index += 4;
	
	  /* Four bytes for player score */
	  /* This is tricky... it can be negative */
	  quaqut_utils_bufferize4bytes( buffer , quaqut.buffer_reply[request] , index , 1 );
	  index += 4;
	
	
	  /* Four bytes: these are mixed for player team and stats ID;
	     if we understood this correctly, the last byte contains
	     information on the team, the rest will be straight ignored */
	  index += 3;
	
          if ( quaqut.buffer_reply[request][index] == 0x20 ) strncat( buffer , "R\n" , 2 );
	  else if ( quaqut.buffer_reply[request][index] == 0x40 ) strncat( buffer , "B\n" , 2 );
	  else strncat( buffer , "/\n" , 2 );
	  index += 1;
	
	  /* On to the next player :) */
	  entrynum = entrynum + 5;
        }
      
        strncat( buffer , "\0" , 1 ); 
      
        /* We copy our translated output from temporary buffer to quaqut buffer */
        strncpy( quaqut.buffer_reply[request] , buffer , 2047 );
	quaqut.buffer_reply[request][2047] = '\0';
      
        /* We need to make a count of lines for gameinfo, stored in entrynum */
        quaqut.buffer_reply_entriesnum[request] = entrynum;
      }
      
      break;
    }
    
  }
}
