/***************************************************************************
 *   Copyright (C) 2004 by Giacomo Lozito                                  *
 *   city_hunter@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#ifdef _WIN_32_
  #include <winsock2.h>
#else
  #include <sys/types.h>
  #include <sys/time.h>
  #include <unistd.h>
  #include <sys/socket.h>
  #include <sys/select.h>
  #include <netinet/in.h>
  #include <netdb.h>
  #include <fcntl.h>
#endif

#define QUAQUT_QUERY_STANDARD_PORT		7778
#define QUAQUT_QUERY_STANDARD_COMMAND		'a'
#define QUAQUT_QUERY_STANDARD_TIMEOUT_SEC	1
#define QUAQUT_QUERY_STANDARD_TIMEOUT_DECSEC	0
/* Max timeout is defined in tenth of seconds, so it is 99.9 seconds */
#define QUAQUT_QUERY_MAX_TIMEOUT		999
/* Max query interval for daemon mode is defined to 9999 seconds */
#define QUAQUT_DAEMON_MAX_INTERVAL		9999
/* Querying the server more than once in ten seconds seems lame to me */
#define QUAQUT_DAEMON_MIN_INTERVAL		10

#define QUAQUT_OPTION_RAWINFO			1
#define QUAQUT_OPTION_VERSION			2
#define QUAQUT_OPTION_DAEMON			4
#define QUAQUT_OPTION_LOGFILE			8

#define QUAQUT_ERROR_DISPLAYVERSION		-1
#define QUAQUT_ERROR_INVALIDQUERYTIMEOUT	-3
#define QUAQUT_ERROR_INVALIDDAEMONINTERVAL	-4
#define QUAQUT_ERROR_MISSINGFILE		-5
#define QUAQUT_ERROR_MISSINGQUERYTIMEOUT	-6
#define QUAQUT_ERROR_MISSINGDAEMONINTERVAL	-7
#define QUAQUT_ERROR_MISSINGHOSTNAME		-8
#define QUAQUT_ERROR_MISSINGPORT		-9
#define QUAQUT_ERROR_FILEPROBLEM		-10



/* Information display */
int quaqut_display_process( int , char );
void quaqut_display_information( FILE* , int , int );
void quaqut_display_instructions( void );
void quaqut_display_error( int , char* , char* );
void quaqut_display_version( void );
void quaqut_display_information_ut2004( FILE* , int );


/* Protocol manipulation */
void quaqut_protocol_process( int , char );
void quaqut_protocol_translate( int , int );
void quaqut_protocol_ut2004( int );


/* Command line manipulation */
int quaqut_cmdlineparam_process( int , char** );
int quaqut_cmdlineparam_parse( int , char** );
int quaqut_cmdlineparam_checkparamarg( int , int , char** );


/* Network */
void quaqut_network_process( int , char );
void quaqut_network_initsocket( void );
void quaqut_network_shutsocket( void );
void quaqut_network_send( int , int );
void quaqut_network_poll( int );
void quaqut_network_recv( int );
int quaqut_network_lookup( void );
int quaqut_network_initsockaddr( struct sockaddr_in* , const char* , unsigned short int );


/* Handy functions */
void quaqut_utils_cleanup( int );
void quaqut_utils_initvalues( void );
void quaqut_utils_cleanbuffers( void );
char* quaqut_utils_strsep( char* , char );
int quaqut_utils_gethex( unsigned int , ... );
int quaqut_utils_stripn( unsigned char* , int );
int quaqut_utils_isipaddress( const char* );
int quaqut_utils_bufferize2bytes( unsigned char* , unsigned char* , int );
int quaqut_utils_bufferize4bytes( unsigned char* , unsigned char* , int , int );
int quaqut_utils_bufferizestring( unsigned char* , unsigned char* , int );


struct {

  char command;
  unsigned int options;
  char* server_host;
  unsigned short int server_port;
  unsigned int recv_timeout_sec;
  unsigned int recv_timeout_decsec;
  unsigned int daemon_interval_sec;
  
  char* output_filename;
  
  int socket;
  struct sockaddr_in server_name;
  
  unsigned char buffer_reply[3][2048];
  int buffer_reply_size[3];
  int buffer_reply_entriesnum[3];
  
} quaqut;
